SystemOrganization addCategory: #'Seaside-Pharo-Security'!

WAKeyGenerator subclass: #WANoSecureKeyGeneratorAvailable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Security'!

!WANoSecureKeyGeneratorAvailable commentStamp: 'pmm 7/20/2014 18:06' prior: 0!
I never generate a key, I always signal an error. I am used as a random key generator when a secure key generator has been installed but could not be seeded. We would rather not generate a key than silently generate insecure ones.!

!WANoSecureKeyGeneratorAvailable methodsFor: 'public' stamp: 'pmm 7/20/2014 18:04'!
keyOfLength: anInteger
	self error: 'no secure key generator availble because seeding failed'! !

WAKeyGenerator subclass: #WASecureKeyGenerator
	instanceVariableNames: 'mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Pharo-Security'!

WASecureKeyGenerator class
	instanceVariableNames: 'generator fortuna sha1'!

!WASecureKeyGenerator commentStamp: 'pmm 7/20/2014 18:42' prior: 0!
I generate cryptographically secure pseudorandom key. I delegate to a random number generator.

It is important that I seed myself automatically on image start up. Otherwise if an image is saved and afterwards started twice all images generate the same random numbers. Also I must not use the system time for seeding as multiple images could be started at the same time. Currently seeding uses /dev/urandom and therefore is only available un Unix environments. If I can't successfully seed myself I install a key generator that always fails.!

WASecureKeyGenerator class
	instanceVariableNames: 'generator fortuna sha1'!

!WASecureKeyGenerator class methodsFor: 'public' stamp: 'pmm 7/20/2014 18:16'!
beFortuna
	generator := #fortuna! !

!WASecureKeyGenerator class methodsFor: 'public' stamp: 'pmm 7/20/2014 18:16'!
beSha1
	generator := #sha1! !

!WASecureKeyGenerator class methodsFor: 'private' stamp: 'pmm 7/20/2014 18:12'!
canSeed
	^ (FileDirectory forFileName: self seedFile) exists! !

!WASecureKeyGenerator class methodsFor: 'public' stamp: 'pmm 7/20/2014 18:28'!
fortuna
	^ fortuna! !

!WASecureKeyGenerator class methodsFor: 'private' stamp: 'pmm 7/20/2014 18:12'!
generateKey: keySize
	^ FileStream oldFileNamed: self seedFile do: [ :stream |
		stream binary; next: keySize ]! !

!WASecureKeyGenerator class methodsFor: 'public' stamp: 'pmm 7/20/2014 18:16'!
generator
	^ self perform: generator! !

!WASecureKeyGenerator class methodsFor: 'initialization' stamp: 'pmm 7/20/2014 18:30'!
initialize
	GRPlatform current addToStartUpList: self.
	"in case seeding fails alway fail"
	self installFailing.
	self canSeed ifTrue: [
		self beSha1.
		self seed.
		self install ]! !

!WASecureKeyGenerator class methodsFor: 'public' stamp: 'pmm 7/20/2014 18:18'!
install
	WAKeyGenerator current: (self new
		generator: nil;
		yourself)! !

!WASecureKeyGenerator class methodsFor: 'private' stamp: 'pmm 7/20/2014 18:29'!
installFailing
	WAKeyGenerator current: WANoSecureKeyGeneratorAvailable new! !

!WASecureKeyGenerator class methodsFor: 'private' stamp: 'pmm 7/20/2014 18:23'!
seed
	fortuna := Fortuna key: (self generateKey: 32). "256 bits"
	sha1 := SecureRandom key: (self generateKey: 32)! !

!WASecureKeyGenerator class methodsFor: 'private' stamp: 'pmm 7/20/2014 18:12'!
seedFile
	^ '/dev/urandom'! !

!WASecureKeyGenerator class methodsFor: 'public' stamp: 'pmm 7/20/2014 18:28'!
sha1
	^ sha1! !

!WASecureKeyGenerator class methodsFor: 'system startup' stamp: 'pmm 7/20/2014 18:30'!
startUp
	"in case seeding fails alway fail"
	self installFailing.
	self canSeed ifTrue: [
		self seed.
		self install ]! !

!WASecureKeyGenerator class methodsFor: 'public' stamp: 'pmm 8/16/2014 16:32'!
uninstall
	WAKeyGenerator current: WAKeyGenerator new.
	sha1 := nil.
	fortuna := nil! !

!WASecureKeyGenerator class methodsFor: 'initialization' stamp: 'pmm 7/20/2014 18:25'!
unload
	GRPlatform current removeFromStartUpList: self.
	self uninstall! !

!WASecureKeyGenerator methodsFor: 'initialization' stamp: 'pmm 7/19/2014 12:15'!
initialize
	super initialize.
	mutex := WAMutex new! !

!WASecureKeyGenerator methodsFor: 'public' stamp: 'pmm 7/20/2014 18:31'!
keyOfLength: anInteger
	^ mutex critical: [
		generator := self class generator.
		^ super keyOfLength: anInteger ]! !

!GRPackage class methodsFor: '*seaside-pharo-security' stamp: 'pmm 2/8/2014 16:15'!
seasidePharoSecurity
	^ self new
		name: 'Seaside-Pharo-Security';
		addDependency: 'Seaside-Security';
		url: #seasideUrl;
		yourself! !
WASecureKeyGenerator initialize!
